@@ -868,7 +868,7 @@ static void cs_register_ss_controls(struct ofono_call_settings *cs)
 
 	__ofono_ussd_ssc_register(cs->ussd, "43", cw_ss_control, cs, NULL);
 
-	if (cs->driver->colr_query)
+	if (cs->driver->colr_query != NULL)
 		__ofono_ussd_ssc_register(cs->ussd, "77",
 					clip_cnap_colp_colr_ss, cs, NULL);
 }
@@ -882,7 +882,7 @@ static void cs_unregister_ss_controls(struct ofono_call_settings *cs)
 
 	__ofono_ussd_ssc_unregister(cs->ussd, "43");
 
-	if (cs->driver->colr_query)
+	if (cs->driver->colr_query != NULL)
 		__ofono_ussd_ssc_unregister(cs->ussd, "77");
 }
 
@@ -1388,7 +1388,7 @@ static void call_settings_remove(struct ofono_atom *atom)
 	if (cs == NULL)
 		return;
 
-	if (cs->driver && cs->driver->remove)
+	if (cs->driver != NULL && cs->driver->remove != NULL)
 		cs->driver->remove(cs);
 
 	g_free(cs);
@@ -1454,7 +1454,6 @@ void ofono_call_settings_register(struct ofono_call_settings *cs)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(cs->atom);
 	struct ofono_modem *modem = __ofono_atom_get_modem(cs->atom);
-	struct ofono_atom *ussd_atom;
 
 	if (!g_dbus_register_interface(conn, path,
 					OFONO_CALL_SETTINGS_INTERFACE,
@@ -1472,12 +1471,6 @@ void ofono_call_settings_register(struct ofono_call_settings *cs)
 					OFONO_ATOM_TYPE_USSD,
 					ussd_watch, cs, NULL);
 
-	ussd_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_USSD);
-
-	if (ussd_atom && __ofono_atom_get_registered(ussd_atom))
-		ussd_watch(ussd_atom, OFONO_ATOM_WATCH_CONDITION_REGISTERED,
-				cs);
-
 	__ofono_atom_register(cs->atom, call_settings_unregister);
 }
 
